#!/usr/bin/env python3

import gzip

class Packet(object):
    """docstring for Packet"""
    def __init__(self, byteorder, data):
        super(Packet, self).__init__()
        self.byteorder = byteorder
        self.data = data

    def __str__(self):
        return 'Packet of ' + str(len(self.data)) + ' bytes'


#!/usr/bin/env python3

import sys
if sys.version_info < (3,):
    print('ERROR: Only Python 3 is supported')
    sys.exit(-1)

__title__ = 'pypcappy'
__version__ = '0.0.4c'
__author__ = 'Guy Taylor'

from .pcapNgFile import PcapNgFile



#!/usr/bin/env python3

from ..packet import Packet
from . import AbstractBlock

class EnhancedPacketBlock(AbstractBlock):
    TYPE_IDS = [0x00000006]

    def __init__(self, byteorder, block_type, data):
        super().__init__(byteorder, block_type, data)

    def block_type_name(self):
        return 'Enhanced Packet Block (EPB)'

    @property
    def capture_len(self):
        return int.from_bytes(self.data[20:25], byteorder=self.byteorder, signed=False)

    @property
    def original_len(self):
        return int.from_bytes(self.data[20:25], byteorder=self.byteorder, signed=False) 

    @property
    def packet(self):
        return Packet(self.byteorder, self.data[28:28 + 1 + self.capture_len])


#!/usr/bin/env python3

from importlib import import_module

from .abstract import AbstractBlock

"""
from .sectionHeaderBlock import SectionHeaderBlock
from .customBlock import CustomBlock
from .interfaceDescriptionBlock import InterfaceDescriptionBlock
from .enhancedPacketBlock import EnhancedPacketBlock

BLOCK_TYPES = {
    0x0A0D0D0A: SectionHeaderBlock,
    0x00000001: InterfaceDescriptionBlock,
    0x00000002: ToDoBlock, #'!OBSOLETE! Packet Block',
    0x00000003: ToDoBlock, #'Simple Packet Block (SPB)',
    0x00000004: ToDoBlock, #'Name Resolution Block (NRB)',
    0x00000005: ToDoBlock, #'Interface Statistics Block (ISB)',
    0x00000006: EnhancedPacketBlock,
    0x00000BAD: CustomBlock,
    0x40000BAD: CustomBlock
}
"""


class ToDoBlock(AbstractBlock):
    TYPE_IDS = []

    def __init__(self, byteorder, block_type, data):
        super().__init__(byteorder, block_type, data, skip=True)

    def block_type_name(self):
        return 'TODO Block'

class BlockLibary(object):
	def __init__(self):
		self.blocks = {}

	def add(self, type_id, block):
		if type_id < 0 or type_id > 0xFFFFFFFF:
			raise Exception('Invalid block type id: ' + str(type_id))
		self.blocks[type_id] = block

	def __getitem__(self, type_id):
		if type_id < 0 or type_id > 0xFFFFFFFF:
			raise Exception('Invalid block type id: ' + str(type_id))
		return self.blocks.get(type_id, ToDoBlock)

BLOCK_TYPES = BlockLibary()

for block_name in ['Abstract', 'SectionHeader', 'Custom', 'InterfaceDescription', 'EnhancedPacket']:
	module = import_module('.' + block_name.lower(), package='pypcappy.blocks')
	block = getattr(module, block_name + 'Block')
	for type_id in block.TYPE_IDS:
		BLOCK_TYPES.add(type_id, block)



#!/usr/bin/env python3

from . import AbstractBlock

class SectionHeaderBlock(AbstractBlock):
    TYPE_IDS = [0x0A0D0D0A]

    def __init__(self, byteorder, block_type, data):
        super().__init__(byteorder, block_type, data)

    def block_type_name(self):
        return 'Section Header Block (SHB)'


#!/usr/bin/env python3

import gzip

from .blocks import BLOCK_TYPES
from .blocks.enhancedpacket import EnhancedPacketBlock

class BlockInterator(object):
    def __init__(self, file):
        self._file = file
        self._current_byteorder = None

    def __iter__(self):
        return self

    def __next__(self):
        if len(self._file.peek(1)) == 0:
            raise StopIteration

        # set or maintain the correct byteorder
        is_section_header_block = self._peek(4) == bytes([0x0A, 0x0D, 0x0D, 0x0A])
        if self._current_byteorder == None and not is_section_header_block:
            raise Exception('Invalid file: No SectionHeaderBlock (' + str(self._peek(4)) + ')')
        if is_section_header_block:
            self._peek_and_set_current_byteorder()

        # read the block in full
        block_type = int.from_bytes(self._file.read(4), byteorder=self._current_byteorder, signed=False)
        block_len = int.from_bytes(self._file.read(4), byteorder=self._current_byteorder, signed=False)
        block_data  = self._file.read(block_len - 4 - 4 - 4)
        block_len_tail = int.from_bytes(self._file.read(4), byteorder=self._current_byteorder, signed=False)
        if block_len != block_len_tail:
            raise Exception('Invalid block')

        return BLOCK_TYPES[block_type](self._current_byteorder, block_type, block_data)

    def _peek_and_set_current_byteorder(self):
        byteorder_magic = self._peek(8 + 4)[0x08:0x0C]
        byteorder_little = int.from_bytes(byteorder_magic, byteorder='little', signed=False)
        byteorder_big = int.from_bytes(byteorder_magic, byteorder='big', signed=False)
        if byteorder_little == 0x1A2B3C4D:
            self._current_byteorder = 'little'
        elif byteorder_big == 0x1A2B3C4D:
            self._current_byteorder = 'big'
        else:
            raise Exception('Invalid SectionHeaderBlock Byte-order Magic')

    def _peek(self, size):
        return self._file.peek(size)[:size]

class PacketInterator(object):
    def __init__(self, blocks):
        self._packets = (block.packet for block in blocks if isinstance(block, EnhancedPacketBlock))

    def __iter__(self):
        return self._packets


class PcapNgFile(object):
    """Wrapper around a sinlge pcap-ng file"""
    def __init__(self, file):
        super(PcapNgFile, self).__init__()
        if isinstance(file, str):
            file = open(file, 'rb')
        if file.peek(2)[:2] == bytes([0x1F, 0x8B]):
            file = gzip.open(file, mode='rb')
        self._file = file
    
    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_val, trace):
        self._file.close()

    @property
    def blocks(self):
        return BlockInterator(self._file)

    @property
    def packets(self):
        return PacketInterator(self.blocks)



#!/usr/bin/env python3

class AbstractBlock(object):
    TYPE_IDS = []

    """docstring for PcapNgBlock"""
    def __init__(self, byteorder, block_type, data, skip=False):
        self.byteorder = byteorder
        if block_type not in self.TYPE_IDS and not skip:
            raise Exception('Invalid TypeID for ' + str(self.block_type_name()) + ': ' + str(block_type))
        self.block_type = block_type
        self.data = data

    @property
    def block_type_name(self):
        raise NotImplementedError()

    def __str__(self):
        return 'PcapNgBlock: ' + str(self.block_type_name())


#!/usr/bin/env python3

import sys
if sys.version_info < (3,):
    print('ERROR: Only Python 3 is supported')
    sys.exit(-1)


# To use a consistent encoding
from codecs import open
from os import path, system
import re
import sys

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

with open('pypcappy/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name = 'pypcappy',
    version = version,
    description = 'Pure Python3 PcapNg reader',
    long_description = long_description,
    author = 'Guy Taylor',
    author_email = 'thebigguy.co.uk@gmail.com',
    url = 'https://github.com/TheBiggerGuy/pypcappy',
    packages = find_packages(exclude=['docs', 'tests']),
    package_data={'': ['LICENSE', 'README.md']},
    zip_safe=False,
    install_requires=[],
    tests_require=['pytest'],
    cmdclass = {'test': PyTest},
    keywords = ['pcap'], 
    license = 'MIT',
    classifiers = (
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ),
)
